from aiozoom.components.base import Base


class Dashboard(Base):
    """
    Class describing `Dashboard` logic of Zoom API
    learn more:
    https://marketplace.zoom.us/docs/api-reference/zoom-api/dashboards/
    """
    async def get_meeting_participants(self, meeting_id: str) -> dict:
        """
        Get list of participants with detail information

        Params:
            meeting_id - unique id of meeting
        """
        method = f'metrics/meetings/{meeting_id}/participants'
        return self.base_get_request(method)



from aiozoom.components.meeting import Meeting
from aiozoom.components.dashboard import Dashboard
from aiozoom.components.recording import Recording


class Zoom(Meeting, Dashboard, Recording):
    """
    Main entrypoint for all components
    """
    pass



import typing as tp

from aiozoom.components.base import Base, ZoomException


class Meeting(Base):
    """
    Class describing `Meeting` logic of Zoom API
    learn more:
    https://marketplace.zoom.us/docs/api-reference/zoom-api/meetings/
    """

    async def create_meeting(self, email: str, body: dict) -> dict:
        """
        Create meeting for the given account email

        Params:
            email - address of account
            body - dict with parameters
        See docs for `body` sample
        https://marketplace.zoom.us/docs/api-reference/zoom-api/meetings/meetingcreate
        """
        if not self.check_body(body):
            raise ZoomException('Body has to be a valid python dictionary')

        method = f'users/{email}/meetings'
        return await self.base_post_request(method, json=body)

    async def get_meeting(self, meeting_id: str) -> dict:
        """
        Get meeting information for given `meeting_id`

        Params:
            meeting_id - unique id of meeting
        """
        method = f'meetings/{meeting_id}'
        return await self.base_get_request(method)

    async def stop_meeting(self, meeting_id: str) -> None:
        """
        Force stop the meeting.

        Params:
            meeting_id - unique id of meeting
        """
        method = f'meetings/{meeting_id}'
        body = {'action': 'end'}
        res = await self.base_put_request(method, json=body)
        print(res)

    async def list_meetings(self, email: str) -> dict:
        """
        List all meetings for account

        Params:
            email - address of account
        """
        method = f'users/{email}/meetings'
        return await self.base_get_request(method)

    async def update_meeting(self, meeting_id: str, body: dict) -> dict:
        """
        Update the details of a meeting

        Params:
            meeting_id - unique id of meeting
            body - dict with parameters (see docs for more)
        """
        if not self.check_body(body):
            raise ZoomException('Body has to be a valid python dictionary')

        method = f'meetings/{meeting_id}'
        return await self.base_patch_request(method, json=body)

    @staticmethod
    def check_body(body: tp.Any) -> bool:
        """
        Check if body is valid for request
        """
        return isinstance(body, dict)



# pylint:disable=(missing-function-docstring)
from aiozoom.config import client
from aiozoom.config.auth import Configuration


class Base(Configuration):
    """
    Base class for processing requests
    """
    async def base_get_request(self, method: str, *args, **kwargs) -> dict:
        data = await client.get(method, headers=self._headers(), *args, **kwargs)
        return data

    async def base_post_request(self, method: str, *args, **kwargs) -> dict:
        data = await client.post(
            method, headers=self._headers(), *args, **kwargs)
        return data

    async def base_put_request(self, method: str, *args, **kwargs) -> dict:
        data = await client.put(
            method, headers=self._headers(), *args, **kwargs)
        return data

    async def base_patch_request(self, method: str, *args, **kwargs) -> dict:
        data = await client.patch(
            method, headers=self._headers(), *args, **kwargs)
        return data

    async def base_delete_request(self, method: str, *args, **kwargs) -> dict:
        data = await client.delete(
            method, headers=self._headers(), *args, **kwargs)
        return data

    @staticmethod
    def _headers() -> dict:
        return {
            'Authorization': f'Bearer {Configuration.auth_token}',
            'Content-Type': 'application/json'
        }


class ZoomException(Exception):
    """
    Base Zoom exception
    """
    pass



from aiozoom.components.base import Base


class Recording(Base):
    """
    Class describing `Recording` logic of Zoom API
    learn more:
    https://marketplace.zoom.us/docs/api-reference/zoom-api/cloud-recording/
    """

    async def get_recording(self, meeting_id: str) -> dict:
        """
        Get meeting's recording information.
        """
        method = f'/meetings/{meeting_id}/recordings'
        return await self.base_get_request(method)

    async def list_recordings(self, email: str) -> dict:
        """
        List all cloud recordings related to user
        """
        method = f'/users/{email}/recordings'
        return await self.base_get_request(method)

    async def delete_all_recording_files(self, meeting_id: str) -> dict:
        """
        Delete all recording files of meeting
        """
        method = f'/meetings/{meeting_id}/recordings'
        return await self.base_delete_request(method)

    async def delete_recording_file(
        self, meeting_id: str, recording_id: str) -> dict:
        """
        Delete a sprecific recording file from a meeting
        """
        method = f'/meetings/{meeting_id}/recordings/{recording_id}'
        return await self.base_delete_request(method)



from os import path
from setuptools import setup, find_packages

directory = path.abspath(path.dirname(__file__))
with open(path.join(directory, 'README.md'), encoding='utf-8') as f:
    long_description = f.read()

VERSION = '0.0.7.2'
DESCRIPTION = 'Async library for interaction with Zoom API'
LONG_DESCRIPTION = 'Async library for interaction with Zoom API.\n' \
                    'It allows you to create, get, stop meetings ' \
                    'and you can get statistics of past meeting'

# Setting up
setup(
        name="aiozoom",
        version=VERSION,
        author="Vladislav Isakov",
        author_email="vladisa88@gmail.com",
        url="https://github.com/vladisa88/aiozoom",
        description=DESCRIPTION,
        long_description=long_description,
        long_description_content_type='text/markdown',
        packages=find_packages(),
        install_requires=['aiohttp'],
        keywords=['python', 'zoom', 'async'],
        classifiers= [
            "Development Status :: 3 - Alpha",
            "Intended Audience :: Education",
            "Programming Language :: Python",
            "Programming Language :: Python :: 3",
            "Programming Language :: Python :: 3.8",
            "Programming Language :: Python :: 3.9",
            "Operating System :: OS Independent",
            "Topic :: Internet",
            "Topic :: Office/Business",
            "Topic :: Software Development :: Libraries",
        ]
)



# pylint:disable=(missing-function-docstring)
class ConfigurationError(Exception):
    """
    Specify own exception
    """
    pass


class Configuration:
    """
    Describe auth logic. Now available
    only JWT_TOKEN authentication
    """

    auth_token: str = ''

    def __init__(self):
        self.assert_has_api_key()

    @staticmethod
    def configure(token: str) -> None:
        Configuration.auth_token = token

    def assert_has_api_key(self):
        if not self.auth_token:
            raise ConfigurationError('Token is required!')



# pylint:disable=(missing-function-docstring)
import typing as tp

from aiohttp import ClientSession


BASE_URL = 'https://api.zoom.us/v2'

class Client:
    """
    Async client for making requests
    """

    def __init__(self, base_url: str = BASE_URL) -> None:
        self.base_url = base_url

    async def get(self, method: str, *args: tp.Any, **kwargs: tp.Any) -> tp.Any:
        async with ClientSession() as session:
            async with session.get(f'{self.base_url}/{method}', *args, **kwargs) as response:
                data = await response.json()
                return data

    async def post(self, method: str, *args: tp.Any, **kwargs: tp.Any) -> tp.Any:
        async with ClientSession() as session:
            async with session.post(f"{self.base_url}/{method}", *args, **kwargs) as response:
                data = await response.json()
                return data

    async def put(self, method: str, *args: tp.Any, **kwargs: tp.Any) -> tp.Any:
        async with ClientSession() as session:
            async with session.put(f"{self.base_url}/{method}", *args, **kwargs) as response:
                print(await response.text())
                data = await response.json()
                return data

    async def patch(self, method: str, *args: tp.Any, **kwargs: tp.Any) -> tp.Any:
        async with ClientSession() as session:
            async with session.patch(f"{self.base_url}/{method}", *args, **kwargs) as response:
                data = await response.json()
                return data

    async def delete(self, method: str, *args: tp.Any, **kwargs: tp.Any) -> tp.Any:
        async with ClientSession() as session:
            async with session.delete(f"{self.base_url}/{method}", *args, **kwargs) as response:
                data = await response.json()
                return data

client = Client()
